@@ -1,21 +1,21 @@
 /* example2.c	Example code showing how to use punycode.
+ * Copyright (C) 2002, 2003  Simon Josefsson
  * Copyright (C) 2002  Adam M. Costello
- * Copyright (C) 2002  Simon Josefsson
  *
- * This file is part of Libstringprep.
+ * This file is part of GNU Libidn.
  *
- * Libstringprep is free software; you can redistribute it and/or
+ * GNU Libidn is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
  *
- * Libstringprep is distributed in the hope that it will be useful,
+ * GNU Libidn is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with Libstringprep; if not, write to the Free Software
+ * License along with GNU Libidn; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  */
@@ -72,7 +72,6 @@ usage (char **argv)
   exit (EXIT_FAILURE);
 }
 
-
 static void
 fail (const char *msg)
 {
@@ -95,7 +94,7 @@ static const char print_ascii[] =
   "\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n"
   " !\"#$%&'()*+,-./"
   "0123456789:;<=>?"
-  "\x40ABCDEFGHIJKLMNO"
+  "\x40""ABCDEFGHIJKLMNO"
   "PQRSTUVWXYZ[\\]^_" "`abcdefghijklmno" "pqrstuvwxyz{|}~\n";
 
 
@@ -104,7 +103,7 @@ main (int argc, char **argv)
 {
   int status;
   int r;
-  unsigned int input_length, output_length, j;
+  size_t input_length, output_length, j;
   unsigned char case_flags[unicode_max_length];
 
   if (argc != 2)
