@@ -354,10 +354,8 @@ strprep[] =
       STRINGPREP_CONTAINS_PROHIBITED},
   { "XMPP resource profile on same string should work though",
     "foo@bar", "foo@bar", stringprep_xmpp_resourceprep},
-  { "XMPP node profile output",
-    "\xC2\xAA", "\x61", stringprep_xmpp_nodeprep},
-  { "XMPP resource profile output",
-    "\xC2\xAA", "\x61", stringprep_xmpp_nodeprep}
+  { "SASL ANONYMOUS plain mechanism",
+    "simon@josefsson.org", "simon@josefsson.org", stringprep_plain}
 #endif
 };
 
@@ -478,8 +476,6 @@ main (int argc, char *argv[])
 #endif
     }
 
-  free (p);
-
 #if 0
   memset (p, 0, 10);
   stringprep_unichar_to_utf8 (0x3316, p);
@@ -487,6 +483,8 @@ main (int argc, char *argv[])
   puts ("");
 #endif
 
+  free (p);
+
   if (debug)
     printf ("Stringprep self tests done with %d errors\n", error_count);
 
